package {
    
    import flash.display.Shape;
    import flash.geom.Rectangle;
    
    public class Blob extends Shape {
        
        public static const EVENT_DEAD:String = "dead";    // Dispatched when a hero blob gets hit
        
        public static const DEFAULT_SPEED:int = 3;
        public static const DEFAULT_RANGE:int = 170;
        
        public var speed:int = DEFAULT_SPEED;    // In pixels per frame
        
        public var displayName:String = "Blob";
        private var moveFunction:Function;       // cos, sin, something like that
        private var xFudge:int = 0;
        private var yTranslate:int;              // How much to shift y by - usually this is the center of the stage.
        private var range:int = DEFAULT_RANGE;
        
        private var totalRotation:int = 0;    // rotation is 90 at start, so tank faces left
        private var isRotating:Boolean = false;
        
        public function Blob(
                            startX:Number, 
                            startY:Number, 
                            color:Number, 
                            directionIndicator:Boolean = false,
                            moveFunctionParam:Function = null) {
            super();
            
            x = startX;
            
            y = startY;
            yTranslate = y;
            
            var radius:int = 10;
            graphics.lineStyle(1, 0xFFFFFF);
            graphics.beginFill(color, 0.7);
            graphics.drawCircle(0, 0, radius);

            if (directionIndicator) { // only the hero has this
                graphics.lineStyle(1, 0x000000);
                graphics.moveTo(0,0);
                graphics.lineTo(0, -radius);
            }
            else {    // so this is only for the enemies. bit of a hack really.
                xFudge = Math.floor(Math.random() * 100);                
            }
            
            if (moveFunctionParam != null) {
                moveFunction = moveFunctionParam;
                y =  moveFunction((x+xFudge) * Math.PI / 180) * range + yTranslate;
            }
                        
        } // end constructor
        
        public function updatePosition(bounds:Rectangle):void {
            
            x += speed;
            
            // wrap x
            if (x >= bounds.width) {
                x = 0;
            }
            
            y = moveFunction((x+xFudge) * Math.PI / 180) * range + yTranslate;
            
            // block y
            if (y > bounds.height) {
                y = bounds.height;
            }
            if (y < bounds.y) {
                y = 0;
            }
        }
        
        /**
        * Called by the BlobManager when this blob dies.
        * 
        * @param killer The Blob that hit us
        */
        public function youAreDead(killer:Blob):void {
            //animateHit();
            
            var event:DeadEvent = new DeadEvent(EVENT_DEAD, killer.displayName);
            dispatchEvent(event);
        }
        /*
        private function animateHit():void {
            isRotating = true;
            addEventListener(Event.ENTER_FRAME, spin);
        }
        */
        /*
        private function spin(event:Event):void {
            
            if (!isRotating) {
                return;
            }
            
            totalRotation += 7;
            rotation = totalRotation;
            
            if (totalRotation >= 270) {
                isRotating = false;
                totalRotation = 0;
                rotation = -90;
                removeEventListener(Event.ENTER_FRAME, spin);
            }
        }
        */
    } // end class    
    
}