package
{
    import flash.display.Sprite;
    import flash.geom.Rectangle;
        
    public class BlobManager {
        
        public static const MOVE_FUNCTIONS:Array = [Math.cos, Math.sin, Math.tan];
        public static const MOVE_FUNCTION_NAMES:Array = ["cosine", "sine", "tangent"];
         
        private var blobs:Array = new Array();
        private var container:Sprite;
        private var stageBounds:Rectangle;
        private var yPos:int;


        /**
        * Constructor.
        * 
        * @param containerParam The container that holds the created blobs. Usually the top level Sprite.
        * @param stageBoundsParam A rectangle defining the bounds in stage co-ordinates that the blobs should stay within. Usually new Rectangle(0,0,stage.stageWidth,stage.stageHeight). 
        * @param yPosParam The Y position blobs start at. Usually stage.stageHeight / 2. Also the range of the function.
        */ 
        public function BlobManager(containerParam:Sprite, stageBoundsParam:Rectangle, yPosParam:int) {
            this.container = containerParam;
            this.stageBounds = stageBoundsParam;
            this.yPos = yPosParam;
        }

        public function reset():void {
            
            for each (var b:Blob in blobs) {
                container.removeChild(b);
            }
            blobs = new Array();
        }

        /**
        * Creates a new Blob using one of the built in functions, positions it, and attaches it to the container.
        */ 
        public function spawn():void {
            
            var func:Function = MOVE_FUNCTIONS[blobs.length % MOVE_FUNCTIONS.length];
            var displayName:String = MOVE_FUNCTION_NAMES[blobs.length % MOVE_FUNCTION_NAMES.length];
              
            var blob:Blob = new Blob(0, yPos, Math.random() * 16777215, false, func);
            blob.displayName = displayName;
            
            container.addChild(blob);
            blobs.push(blob);
        }

        /**
        * Have any of our blobs hit the testObject.
        * If they have, the blob that hit gets removed, and the testObject gets animateHit called on it.
        */
        public function isCollision(testObject:Blob):Boolean {
            
            var collisionIndex:int = -1;
            
            for (var i:int = 0; i< blobs.length; i++) {
                if ( blobs[i].hitTestObject(testObject) ) {
                    collisionIndex = i;
                    break;
                }
            }
            
            if (collisionIndex != -1) {
                var hitBy:Blob = blobs[collisionIndex];
                container.removeChild(hitBy);
                blobs.splice(collisionIndex, 1);
                
                testObject.youAreDead(hitBy);
                return true;
            }
            else {
                return false;
            }
        }
        
        public function moveAll():void {

            for (var i:int = 0; i< blobs.length; i++) {
                blobs[i].updatePosition(stageBounds);
            }
        }

    }
}