package
{
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import flash.display.StageAlign;
    import flash.display.StageScaleMode;
    import flash.events.Event;
    import flash.events.KeyboardEvent;
    import flash.events.MouseEvent;
    import flash.events.TimerEvent;
    import flash.geom.Rectangle;
    import flash.text.TextField;
    import flash.text.TextFieldAutoSize;
    import flash.text.TextFieldType;
    import flash.text.TextFormat;
    import flash.ui.Keyboard;
    import flash.utils.Timer;

    public class MathDodger extends Sprite {

        [Embed(source="background.jpg")]
        private var backgroundImage:Class;
        private var backImg:DisplayObject;
        private var backImg2:DisplayObject;
        
        private var centerX:int;
        private var centerY:int;
        
        private var timeWidget:TimeWidget;

        private var tank:Blob;
        private var isHalted:Boolean = true;    // Is the game paused?
        
        private var rotateTimer:Timer = new Timer(10, 0);
        private var moveTimer:Timer = new Timer(10, 0);
        
        private var blobManager:BlobManager;
        
        private var playerName:String;
        private var nameInput:TextField;
        private var textFields:Array = new Array();        // Various pieces of text displayed before the game starts
        
        public function MathDodger() {

            super();

            stage.scaleMode = StageScaleMode.NO_SCALE;
            stage.align = StageAlign.TOP_LEFT;
            
            centerX = stage.stageWidth / 2;
            centerY = stage.stageHeight / 2;
            
            // Blob spawning zone
            //graphics.beginFill(0xFF4444);    
            //graphics.drawRect(0, 0, 100, stage.stageHeight);
            //graphics.endFill();

            addBackground();
            addIntroText();
            
            var bounds:Rectangle = new Rectangle(0,0,stage.stageWidth,stage.stageHeight);
            blobManager = new BlobManager(this, bounds, centerY);            
            
            stage.addEventListener(MouseEvent.CLICK, onInitialStageClick);
            stage.addEventListener(KeyboardEvent.KEY_UP, onStartKeypress);
        }
        
        /**
        * Create and display the intro text, and get the users name
        */
        private function addIntroText():void {

            var titleText:TextField = makeText("Math Dodger", 0x6F6F6F);
            titleText.x = centerX - titleText.width / 2;
            titleText.y = 10;
            addChild(titleText);
            textFields.push(titleText);
            
            var nameText:TextField = makeText("Your name:", 0xFFFFFF);
            nameText.x = centerX - nameText.width / 2;
            nameText.y = centerY - nameText.height - 10;            
            addChild(nameText);
            textFields.push(nameText);

            nameInput = makeText("", 0x000000); 
            nameInput.type = TextFieldType.INPUT;
            nameInput.x = centerX;
            nameInput.y = centerY;
            nameInput.backgroundColor = 0xFFFFFF;
            nameInput.background = true;            
            addChild(nameInput);
        }

        /**
        * Create and display the backdrop
        */
        private function addBackground():void {
            backImg = new backgroundImage();
            backImg.x = -backImg.width + stage.stageWidth;
            addChild(backImg);
            
            backImg2 = new backgroundImage();
            backImg2.x = backImg.x - backImg2.width;
            addChild(backImg2);
                        
        }

        private function makeText(str:String, color:Number, size:int = 40):TextField {
            
            var format:TextFormat = new TextFormat();
            format.font = "Verdana";
            format.color = color;
            format.size = size;
            
            var result:TextField = new TextField();
            result.autoSize = TextFieldAutoSize.CENTER;
                        
            result.defaultTextFormat = format;
            result.text = str;
            
            return result;            
        }

        /**
        * Focus on the name input field when the mouse is clicked.
        */
        private function onInitialStageClick(event:MouseEvent):void {
            stage.focus = nameInput;
        }

        /**
        * Run when the user presses a key during the intro screen - use to detect submitting the name field.
        */
        private function onStartKeypress(event:KeyboardEvent):void {
            
            if ( event.keyCode == Keyboard.ENTER ) {
                playerName = nameInput.text;
                stage.removeEventListener(KeyboardEvent.KEY_UP, onStartKeypress);
                stage.removeEventListener(MouseEvent.CLICK, onInitialStageClick);
                startGame();            
            }
        }

        private function startGame():void {
            
            // Cleanup the intro fields
            for each (var tf:TextField in textFields) {
                removeChild(tf);
            }
            textFields = new Array();
            
            if ( nameInput ) {
                removeChild(nameInput);
                nameInput = null;
            }

            // The center background clock - create before the blob so that it's underneath
            startTimer();            
            
            // The first enenmy
            blobManager.spawn();
            
            
            // Our hero
            tank = new Blob(centerX, centerY, 0xFFFFFF, true);
            tank.rotation = -90;
            tank.speed = 2;
            tank.addEventListener(Blob.EVENT_DEAD, onHeroDead);
            addChild(tank);
            
            stage.focus = this;
            
            // GO !
            stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
            stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
            addEventListener(Event.ENTER_FRAME, onEnterFrame);
            isHalted = false;            
        }

        private function startTimer():void {
            
            if (timeWidget) {
                timeWidget.reset();
            }
            else {    // first time we play
                timeWidget = new TimeWidget();             
                timeWidget.x = centerX - timeWidget.width / 2;
                timeWidget.y = centerY - timeWidget.height / 2;
                timeWidget.addEventListener(TimeWidget.TICK, onTick);
                addChild(timeWidget);
            }
            
            timeWidget.start();
        }

        private function onEnterFrame(event:Event=null):void {
            
            if (isHalted) {
                return;
            }
            
            backImg.x += 1;
            backImg2.x += 1;
            
            if ( backImg.x > stage.stageWidth ) {
                backImg.x = backImg2.x - backImg.width;
            }
            if (backImg2.x > stage.stageWidth ) {
                backImg2.x = backImg.x - backImg2.width;
            }
            
            if (blobManager.isCollision(tank)) {    // If this is true BlobManager will tell the hero, and that will dispatch EVENT_DEAD 
                isHalted = true;
                return;
            }
            
            blobManager.moveAll();
        }
         
        private function onTick(event:Event):void {

            if (isHalted) {
                return;
            }
            
            var seconds:int = timeWidget.seconds;
            if ( seconds % 10 == 0 ) {
                blobManager.spawn();
            }
        }         

        private function onHeroDead(event:DeadEvent):void {

            isHalted = true;
            timeWidget.stop();
            blobManager.reset();
            removeChild(tank);
            
            var hitMsg:TextField = makeText("Hit by a "+ event.killerName +" function!", 0x006FFF, 30);
            hitMsg.x = centerX - hitMsg.width / 2;
            hitMsg.y = 15;            
            addChild(hitMsg);
            textFields.push(hitMsg);
            
            var timeMsg:TextField = makeText( playerName +", you score "+ timeWidget.seconds, 0xFFFFFF, 30 );
            timeMsg.x = centerX - timeMsg.width / 2;
            timeMsg.y = 20 + timeMsg.height;
            addChild(timeMsg);
            textFields.push(timeMsg);
            
            var continueMsg:TextField = makeText("Click to play again", 0x006FFF);
            continueMsg.x = centerX - continueMsg.width / 2; 
            continueMsg.y = centerY;
            addChild(continueMsg);
            textFields.push(continueMsg);
            
            stage.addEventListener(MouseEvent.CLICK, onContinue);
        }

        private function onContinue(event:MouseEvent):void {
            stage.removeEventListener(MouseEvent.CLICK, onContinue);
            startGame();
        } 

        private function rotateLeft(event:TimerEvent=null):void {
            tank.rotation -= tank.speed;
            if (event) {
                event.updateAfterEvent();
            }            
        }
        
        private function rotateRight(event:TimerEvent=null):void {
            tank.rotation += tank.speed;
            if (event) {
                event.updateAfterEvent();
            }            
        }
        
        private function saneX(x:int):int {
            if (x < 0) {
                return 0;
            }
            else if (x > stage.stageWidth) {
                return stage.stageWidth;
            }
            else {
                return x;
            }
        }
        
        private function saneY(y:int):int {
            if ( y < 0 ){
                return 0;
            }
            else if ( y > stage.stageHeight ) {
                return stage.stageHeight;
            }
            else {
                return y;
            }
        }
        
        private function goForward(event:TimerEvent=null):void {
            
            var newX:int = tank.x + Math.sin(Math.PI / 180 * tank.rotation) * tank.speed; 
            var newY:int = tank.y - Math.cos(Math.PI / 180 * tank.rotation) * tank.speed;
            
            tank.x = saneX(newX);
            tank.y = saneY(newY); 
            
            if (event) {
                event.updateAfterEvent();
            }
        }
        
        private function goBackward(event:TimerEvent=null):void {

            var newX:int = tank.x - Math.sin(Math.PI / 180 * tank.rotation) * tank.speed;
            var newY:int = tank.y + Math.cos(Math.PI / 180 * tank.rotation) * tank.speed;
            
            tank.x = saneX(newX);
            tank.y = saneY(newY);             
            
            if (event) {
                event.updateAfterEvent();
            }            
        }
        
        private function onKeyDown(event:KeyboardEvent):void {
            event.preventDefault();
            
            switch (event.keyCode) {
                case Keyboard.LEFT:
                    
                    if (rotateTimer.running) {
                        return;
                    }
                    rotateLeft();
                    rotateTimer.addEventListener(TimerEvent.TIMER, rotateLeft);
                    rotateTimer.start();
                    break;
                    
                case Keyboard.RIGHT:
                
                    if (rotateTimer.running) {
                        return;
                    }
                    rotateRight();
                    rotateTimer.addEventListener(TimerEvent.TIMER, rotateRight);
                    rotateTimer.start();
                    break;
                    
                case Keyboard.UP:
                
                    if (moveTimer.running) {
                        return;
                    }
                    goForward();
                    moveTimer.addEventListener(TimerEvent.TIMER, goForward);
                    moveTimer.start();
                    break;
                    
                case Keyboard.DOWN:
                
                    if (moveTimer.running) {
                        return;
                    }                
                    goBackward();
                    moveTimer.addEventListener(TimerEvent.TIMER, goBackward);
                    moveTimer.start();
                    break;
            }
            
        }
        
        private function onKeyUp(event:KeyboardEvent):void {
            event.preventDefault();
            
            switch (event.keyCode) {
                case Keyboard.LEFT:
                    rotateTimer.stop();
                    rotateTimer.removeEventListener(TimerEvent.TIMER, rotateLeft);
                    break
                case Keyboard.RIGHT:
                    rotateTimer.stop();
                    rotateTimer.removeEventListener(TimerEvent.TIMER, rotateRight);                    
                    break;
                case Keyboard.UP:
                    moveTimer.stop();
                    moveTimer.removeEventListener(TimerEvent.TIMER, goForward);
                    break;
                case Keyboard.DOWN:
                    moveTimer.stop();
                    moveTimer.removeEventListener(TimerEvent.TIMER, goBackward);                    
                    break;
            }
        }

        
    } // end class
    
}
